// CadThumbsListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "ThumbsListCtrl.h"
#include "previewREaders.h"

#include <vector>
#include <string>
#include <algorithm>


int SearchDirectory(std::vector<std::string> &refvecFiles,
                    const std::string        &refcstrRootDirectory,
                    const std::string        &refcstrExtension,
                    bool                     bSearchSubdirectories = true)
{
	std::string     strFilePath;             // Filepath
	std::string     strPattern;              // Pattern
	std::string     strExtension;            // Extension
	HANDLE          hFile;                   // Handle to file
	WIN32_FIND_DATA FileInformation;         // File information


	strPattern = refcstrRootDirectory + "\\*.*";

	hFile = ::FindFirstFile(strPattern.c_str(), &FileInformation);
	if (hFile != INVALID_HANDLE_VALUE)	{
		do	{
			if (FileInformation.cFileName[0] != '.') {
				strFilePath.erase();
				strFilePath = refcstrRootDirectory + "\\" + FileInformation.cFileName;
				if (FileInformation.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
					if(bSearchSubdirectories) {
						// Search subdirectory
						int iRC = SearchDirectory(refvecFiles,
									strFilePath,
									refcstrExtension,
									bSearchSubdirectories);
						if(iRC)
							return iRC;
					}
				} else	{
					// Check extension
					strExtension = FileInformation.cFileName;
					strExtension = strExtension.substr(strExtension.find(".") + 1);

					if (stricmp(strExtension.c_str(), refcstrExtension.c_str()) == 0) {
						// Save filename
						refvecFiles.push_back(strFilePath);
					}
				} //else
			} //if
	} while (::FindNextFile(hFile, &FileInformation) == TRUE);

	// Close handle
	::FindClose(hFile);

	DWORD dwError = ::GetLastError();
	if (dwError != ERROR_NO_MORE_FILES)
		return dwError;
	}

	return 0;
}

#define	THUMBNAIL_WIDTH		80
#define	THUMBNAIL_HEIGHT	62

// CThumbsListCtrl

IMPLEMENT_DYNAMIC(CThumbsListCtrl, CListCtrl)
CThumbsListCtrl::CThumbsListCtrl()
{
	m_pRdr = new CPicDataReader;
}

CThumbsListCtrl::~CThumbsListCtrl()
{
	delete m_pRdr;
}


BEGIN_MESSAGE_MAP(CThumbsListCtrl, CListCtrl)
	ON_WM_CREATE()
END_MESSAGE_MAP()

void CThumbsListCtrl::showFilesFromFolder(const CString& strFold)
{
	CWaitCursor wait; // crete a sand watch cursor
	
	if(strFold == m_strLastFold) return; //no folder changed so display the same content
	m_strLastFold = strFold;
    //folder changed so empty the list ctrl
	DeleteAllItems();

	std::vector<std::string> arFiles;
	SearchDirectory(arFiles, (LPCSTR)strFold, (LPCSTR)m_strFileExt, false);
	if (arFiles.empty()) return; // no files to preview
	
	// set the length of the space between thumbnails
	// you can also calculate and set it based on the length of your list control
	int nGap = 4;

	// hold the window update to avoid flicking
	SetRedraw(FALSE);

	// reset our image list
	for (int i=m_iList.GetImageCount()-1; i>=0; i--)
		m_iList.Remove(i);			

	// remove all items from list view
	if (GetItemCount() != 0)
		DeleteAllItems();

	// set the size of the image list
	m_iList.SetImageCount(arFiles.size());
	i = 0;

	CBitmap mfcBmp; //mfc bitmap that we must put in the image list
	Bitmap* pGdiBmp = NULL; //gdi+ bitmap we get from the readers
	Bitmap* pThumb = NULL; //thumbnail
	HBITMAP hBmp;

	CPoint pt;
	for(i=0; i<(int)arFiles.size(); i++) {
		m_pRdr->setFile(arFiles[i].c_str());
		pGdiBmp = m_pRdr->preview();
		ASSERT(pGdiBmp!=NULL);
		if(pGdiBmp->GetHeight() == THUMBNAIL_HEIGHT && pGdiBmp->GetWidth() == THUMBNAIL_WIDTH) {
			pThumb = pGdiBmp;  
		} else {
			pThumb = (Bitmap*)pGdiBmp->GetThumbnailImage(THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT);
			ASSERT(pThumb!=NULL);			
		}
		pThumb->GetHBITMAP(Color(255,255,255), &hBmp);		
		
		mfcBmp.Attach(hBmp);
        // add bitmap to our image list
		m_iList.Replace(i, &mfcBmp, NULL);

		// put item to display
		// set the image file name as item text
		CString str = arFiles[i].c_str();
		str = str.Right(str.GetLength() - str.ReverseFind('\\') - 1);
		InsertItem(i, str, i);
		
		mfcBmp.Detach();
		if(pThumb!=pGdiBmp)
			delete pThumb; //delete it only if it was creted separately
		delete pGdiBmp;		
		::DeleteObject(hBmp);
		pGdiBmp=pThumb=NULL;
		hBmp=NULL;
	}

	SetRedraw(TRUE);  

	SetFocus();
	SetItemState(0, LVIS_SELECTED|LVIS_FOCUSED, LVIS_SELECTED|LVIS_FOCUSED);
} //showFilesFromFolder


int CThumbsListCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CListCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_iList.Create(THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT, ILC_COLOR24, 0, 1);
	SetImageList(&m_iList, LVSIL_NORMAL);

	return 0;
}

CString CThumbsListCtrl::getCurSel()
{
	POSITION pos = GetFirstSelectedItemPosition();
	int nItem = GetNextSelectedItem(pos);
	return GetItemText(nItem, 0);
}
